from typing import *
import random
import math

'''
  Python helper function to generate a 2d array.
  :param rows The number of rows
  :param columns The number of columns
'''


def make2dArray(rows: int, columns: int):
    M = [[0 for i in range(columns)] for j in range(rows)]
    return M


class Matrix:

    def __init__(self, arr):
        self.matrix = [[]]
        self.columnCount = 0
        self.isSquare = False
        # A seed for the random matrix generate
        self.seed = 0
        self.arr = arr

        '''
         Creates an mxn matrix.
         :param arr A two-dimensional array containing the elements of the matrix.
        '''
        # Checks that the matrix is not missing a value at the ij-th position
        for i in range(len(self.arr)):
            if len(self.arr[i]) != len(self.arr[0]):
                print(
                    "Matrix cannot be created because {} does not match row 0's length.".format(i))
                raise NameError("Error: Invalid Matrix Format")

        self.matrix = self.arr
        self.rowCount = len(self.matrix)
        self.columnCount = len(self.matrix[0])
        self.isSquare = self.rowCount == self.columnCount

    def colunmCount(self):
        return self.columnCount

    def show(self) -> None:
        print(self.matrix)

        '''
        Calculates the size of the (m x n) matrix.
        :return A tuple [m, n] containing the size of the matrix, where m is
        the number of rows and n in the number of columns.
        '''

    def size(self):
        return [self.rowCount, self.columnCount]

        '''
         Generates the identity matrix of size s.
         param size The size of the identity matrix
         return The identity matrix of size s.
        '''
    @staticmethod
    def identity(size: int) -> callable:
        M = make2dArray(size, size)
        for i in range(0, size):
            for j in range(0, size):
                M[i][j] = 1 if (i == j) else 0

        '''
         Obtains the element of the mxn matrix at position i,j.
         :param i The rwo position of the element.
         :param j The column position of the element.
         :return The element at position i,j.
        '''

    def getElement(self, i: int, j: int) -> callable:
        return self.matrix[i][j]

    '''
    Clones the matrix.
    :return The cloned matrix.
    '''

    def cloneMatrix(self) -> callable:
        M = [_ for _ in range(self.rowCount)]
        for i in range(0, self.rowCount):
            M[i] = self.matrix[i]
        return Matrix(M)
    '''
      Converts the matrix into Row-Reduced Echelon Form (RREF).
      This algorithm is a translation from the pseudo-code found at wikipedia.org
      (https://en.wikipedia.org/wiki/Gaussian_elimination#Pseudocode) into python code.
      :return A new Matrix with the row-reduced version of this matrix.
    '''

    def RRFE(self) -> callable:
        M = self.cloneMatrix()
        h = 0
        k = 0

        while (h < M.rowCount and k < M.columnCount):
            i_max = h
            i = 1
            for i in range(0, M.rowCount):
                if (abs(M.getElement(i, k)) > abs(M.getElement(i_max, k))):
                    i_max = i
                if (M.getElement(i_max, k) == 0):
                    k = k + 1
                else:
                    temp = M.matrix[i_max]
                    M.matrix[i_max] = M.matrix[h]
                    M.matrix[h] = temp

                    s = 1 / M.getElement(h, k)
                    for i_max in range(0, M.columnCount):
                        M.matrix[h][i_max] *= s
                    for i in range(0, M.rowCount):
                        if i != h:
                            t = M.getElement(i, k)
                            for i_max in range(0, M.columnCount):
                                M.matrix[i][i_max] -= t * \
                                    M.getElement(h, i_max)
                    h += 1
                    k += 1
        return M

    '''
    Adds the number n to each element of the matrix.
    :param n The number to add to each element of the matrix.
    '''

    def elAdd(self, n: int) -> callable:
        for i in range(0, self.rowCount):
            for j in range(0, self.columnCount):
                self.matrix[i][j] += n  # NOTE: Add n to each element
        return self
    '''
    Subtracts the number n from each element of the matrix.
    :param n The number to subtract from each element of the matrix.
    '''

    def elSubtract(self, n: int) -> callable:
        return self.elAdd(-1 * n)

    '''
    Multiplies each element of the matrix by n.
    :param n The number by which element of the matrix will be multiplied.
    '''

    def elMultiply(self, n: int) -> callable:
        for i in range(0, self.rowCount):
            for j in range(0, self.columnCount):
                self.matrix[i][j] *= n  # NOTE: Multiply n to each element
        return self

    '''
    Divides each element of the matrix by the number n.
    :param n The number by which each element of the matrix will be divided.
    '''

    def elDivide(self, n: int) -> callable:
        # call elMultiply method to Multiply by 1/n each element
        return self.elMultiply(1 / n)

    '''
    Computes the transposed version (M^T) of this matrix.
    :return The transposed version (M^T) of this matrix.
    '''

    def T(self) -> callable:
        M = make2dArray(self.columnCount, self.rowCount)
        for i in range(0, self.columnCount):
            for j in range(0, self.rowCount):
                M[i][j] = self.getElement(j, i)
            return Matrix(M)

    '''
    Retrieves the row at position r of the matrix.
    :param r The position of the row to be retrieved.
    :return A new Vect whose elements are the entries at row r of the matrix.
    '''

    def getColumn(self, c: int) -> callable:

        column = [_ for _ in range(0, self.rowCount)]
        for i in range(0, self.rowCount):
            column[i] = self.getElement(i, c)
        return column

    '''
    Performs the augmentation (A.K.A. concatenation) of this matrix and the matrix M.
    :param M The matrix to be concatenated to this matrix.
    :return A new Matrix whose elements are the augmentation of this matrix and the matrix M.
    '''

    def augement(self, M: callable) -> callable:
        if self.rowCount != M.rowCount:
            print("Matrices must contai the same number of rows.")
            raise NameError("Error: Matrix row number error")
        augM = make2dArray(self.rowCount, self.columnCount + M.columnCount)
        for i in range(0, self.rowCount):
            for j in range(0, self.columnCount):
                augM[i][j] = self.getElement(i, j) if (
                    j < self.columnCount) else M.getElement(i, j - self.columnCount)
        return Matrix(augM)

    '''
    Computes the inverse of this matrix.
    :return The inverse of the matrix.
    :throws Error If the matrix is not square.
    :throws Error If the matrix has a zero-determinant.
    '''

    def inverse(self) -> callable:
        if self.isSquare == False:
            print("Matrix inverse cannot be computed because the matrix is not square")
            raise NameError("Error: Matrix not square")
        determinant = self.det()
        if (determinant == 0):
            print("Matrix inverse cannot be computed because matrix is singular")
            raise NameError("Error: Matrix is singular")
        return self.adjugate().elDivide(self.det())

    '''
     Computes the determinant of the the matrix.
     :return The determinant of the matrix.
     :throws Error If the matrix is not square.
    '''

    def det(self) -> int:
        if self.isSquare == False:
            print(
                "Matrix determinant cannot be computed because the matrix is not square")
            raise NameError("Error: Matrix not square")
        if self.rowCount == 1 and self.columnCount == 1:
            return self.getElement(0, 0)
        if self.rowCount == 2 and self.columnCount == 2:
            return (self.getElement(0, 0) * self.getElement(1, 1)) - (self.getElement(0, 1) * self.getElement(1, 0))
        determinant = 0
        for j in range(0, self.getColumn):
            determinant += math.pow(-1, j) * self.getElement(0,
                                                             j) * self.subMatrix(0, j).det()
        return determinant

    '''
     Computes the sub-matrix of this matrix when the row r and the column c are ignored from this matrix.
     :param r The row to ignore.
     :param c The column to ignore.
     :return A new Matrix whose elements are the elements of the sub-matrix of this matrix when the row r and the
     column c are ignored from this matrix.
    '''

    def subMatrix(self, r: int, c: int) -> callable:
        M = make2dArray(self.rowCount - 1, self.columnCount - 1)
        row = 0
        column = 0
        for i in range(0, self.rowCount):
            row = i if (i < r) else i - 1
            for j in range(0, self.columnCount):
                column = j if (j < c) else j - 1
                if (i != r and j != c):
                    M[row][column] = self.getElement(i, j)
        return Matrix(M)

    '''
      Computes the matrix of minors of this matrix.
      :return A new Matrix whose elements are the elements of the matrix of minors for this matrix.
    '''

    def mirorsMatrix(self) -> callable:
        if self.isSquare == False:
            print(
                "Matrix inverse cannot be computed because the matrix is not square")
            raise NameError("Error: Matrix is not square")
        MofMinors = make2dArray(self.rowCount, self.columnCount)
        for i in range(0, self.rowCount):
            for j in range(0, self.columnCount):
                MofMinors[i][j] = self.subMatrix(i, j).det()
        return Matrix(MofMinors)

    '''
      Computes the matrix of cofactors for this matrix.
      :return A new Matrix whose elements are the elements of the matrix of cofactors for this matrix.
    '''

    def cofactorsMatrix(self) -> callable:
        MofCofactors = self.mirorsMatrix()
        for i in range(0, MofCofactors.rowCount):
            for j in range(0, MofCofactors.columnCount):
                MofCofactors.matrix[i][j] = math.pow(
                    -1, i + j) * MofCofactors.getElement(i, j)
        return MofCofactors

    '''
     Returns the adjugate of this matrix.
     As defined by wikipedia, the adjugate, classical adjoint, or adjunct of
     a square matrix is the transpose of its cofactor matrix
     :return A new Matrix whose elements are the elements of the adjugate of this matrix.
    '''

    def adjugate(self) -> callable:
        return self.cofactorsMatrix().T()

    '''
     Maps each element of this matrix to a new Matrix based on the provided mapping function f.
     :param f The mapping function to be applied to each element of this matrix.
     :return A new Matrix whole elements are the mapped elements of this matrix based on the mapping function f.
    '''

    def map(self, f: callable) -> callable:
        M = make2dArray(self.rowCount, self.columnCount)
        for i in range(0, self.rowCount):
            for j in range(0, self.columnCount):
                M[i][j] = f(self.getElement(i, j))
        return Matrix(M)

    '''
     Multiplies the matrices.
     :param matrices A list of matrices to be multiplied.
     :return The product of all the matrices.
     :throws Error If there is a mismatch between the sizes of the matrices.
     :throws Error If there is only one matrix as argument.
    '''
    @staticmethod
    def multiply(matrices: list()) -> callable:
        if len(matrices) <= 1:
            print("Matrix multiplication can only happen between 2 or more matrices")
            raise NameError("Error: Error to multiply")
        if len(matrices) == 2:
            A = Matrix(matrices[0])
            B = Matrix(matrices[1])

            if A.colunmCount() != B.rowCount:
                e = "matrix size mismatch. Matrix A (size: {0}) cannot be multiplied by matrix B (size: {1})".format(
                    (A.rowCount + "x" + A.colunmCount()), (B.rowCount + "x" + B.colunmCount()))
                print(e)
                raise NameError("Error: Matrix cannot be multiplied")
            else:
                M = make2dArray(A.rowCount, B.colunmCount())
                for i in range(0, A.rowCount):
                    currentVectorA = A.getRow(i)
                    for j in range(0, B.colunmCount()):
                        currentVectoB = B.getColumn(j)
                        # Calulate their dot-product and palce the result
                        # in the appropriate position of the new matrix.
                        M[i][j] = currentVectorA.dot(currentVectoB)
                return Matrix(M)
        else:
            M = self.multiply(matrices[0], matrices[1])
            for i in range(0, (len(matrices) - 2)):
                M = self.multiply(M, matrices[i + 2])
                return M
    '''
     Generates an <code>mxn</code> matrix with random elements between zero and one.
     :param m The number of rows.
     :param n The number of columns.
     :return A new Matrix whose elements are random doubles between zero and one.
    '''
    @staticmethod
    def rand(m: int, n: int) -> callable:
        matrixEls = make2dArray(m, n)
        for i in range(0, m):
            for j in range(0, n):
                matrixEls[i][j] = random.random()
        return Matrix(matrixEls)

    '''
    Generates an <code>mxn</code> zero-matrix.
    :param m The number of rows.
    :param n The number of columns.
    :return A new Matrix whose elements are all zeros.
    '''
    @staticmethod
    def zeros(m: int, n: int) -> callable:
        matrixEls = make2dArray(m, n)
        for i in range(0, m):
            for j in range(0, n):
                matrixEls[i][j] = 0
        return Matrix(matrixEls)
    '''
     Computes the sparsity of the matrix.
     As defined by Wikipedia.org, a sparse matrix or sparse array is a matrix in which most of the elements are zero.
     :return A float representing the sparsity of the matrix.
    '''

    def sparsity(self) -> float:
        zeroCount = 0
        for i in range(0, self.rowCount):
            for j in range(0, self.columnCount):
                if self.getElement(i, j) == 0:
                    zeroCount += 1
        return (zeroCount / (self.rowCount * self.columnCount))

    '''
      Computes whether a matrix is sparse.
      :return True if the sparsity of the matrix is greater than or equal to 0.5
    '''

    def isSparse(self) -> bool:
        return self.sparsity() >= 0.5

    '''
     Computes the density of the matrix.
     :return A float representing the density of the matrix.
    '''

    def density(self) -> int:
        return 1 - self.sparsity()

    '''
      Computes whether a matrix is dense.
      :return True if the density of the matrix is greater than or equal to 0.5
    '''

    def isDense(self) -> bool:
        return self.density() >= 0.5

    '''
    Determines whether the matrix is a column matrix or not.
    :return True if the matrix is a column matrix. False otherwise.
    '''

    def isColumn(self):
        return self.columnCount == 1

    '''
     Determines whether the matrix is a row matrix or not.
     :return True if the matrix is a row matrix. False otherwise.
    '''

    def isRow(self):
        return self.rowCount == 1
    '''
     Get the row  of one matrix
     :return Return a new Object Vect with the row.
    '''

    def getRow(self, r: int) -> callable:
        from Vect import Vect
        return Vect(self.matrix[r])



from typing import *


class Calculus():

    def __init__(self):
        pass
    '''
    Compute the instantaneous rate of change at x
    using the slope of the target line that passes through two points of the function f with h=0.0000000001.
    :param x The x-value at which the instantaneous rate of change will be calculated.
    :param f The function to calculate the instantaneous rate of change.
    :return The instantaneous rate of change at x for the function f.
    '''
    @staticmethod
    def nDeriv(x: float, f) -> float:
        h = 0.0000000001
        return (f(x + h) - f(x - h)) / (2 * h)

    '''
      Computes the area under a curve in the xy-plane
      using the method of trapezoidal approximation.
      :param l The lower limit of the integral
      :param u The upper limit of the integral
      :param f The function whose area will be approximated
      :return The approximate area under the curve f from x=l to x=u
     '''

    @staticmethod
    def fnInt(l: float, u: float, f) -> float:
        if ((u - l) == 0):
            return 0
        # The number of trapezoids is dynamically
        # adjusted based in the size of the interval
        h = round(100000 * (u - l)) + 100000
        # Total computed
        area = 0
        #Change in x
        deltax = (u - l) / h
        # starting x-value
        x = l

        # Calulate the are of each trapezoid and
        # aggregate it to the total calculated aggregate
        i = 0
        while i <= h:
            area += (f(x) + f(x + deltax) / 2) * deltax
            # The next x-value
            x += deltax
            i += 1
        return area

    '''
    Checks if a function is increasing on an interval [a, b].
    :param a The lower bound of the interval.
    :param b The upper bound of the interval.
    :param f The function to be tested.
    :return True if the function is monotone increasing on [a, b], false otherwise.
    '''

    @staticmethod
    def isIncreasing(a: float, b: float, f) -> bool:
        testN = b - a
        currentTest = 0

        while currentTest <= testN:
            # If the derivative is less than zero for any x in
            # the interval [a, b], then the function would be decreasing,
            # so we return false.
            if (Calculus().nDeriv(a + currentTest, f) < 0):
                return False
            currentTest += 1.0 / 300  # Perfroms 300 tests per unit destance
        return True

    '''
      Checks if a function is decreasing on an interval [a, b].
      param: a The lower bound of the interval.
      param: b The upper bound of the interval.
      param: f The function to be tested.
      return: True if the function is monotone decreasing on [a, b], false otherwise.
     '''
    @staticmethod
    def isDecreasing(a: float, b: float, f) -> bool:
        testN = b - a
        currentTest = 0

        while currentTest <= testN:
            # If the derivative is less than zero for any x in
            # the interval [a, b], then the function would be decreasing,
            # so we return false.
            if (Calculus().nDeriv(a + currentTest, f) > 0):
                return False
            currentTest += 1.0 / 300  # Perfroms 300 tests per unit destance
        return True



from typing import *
import math
import random
# from VectorCoords import VectorCoords


class Vect():

    def __init__(self, elements: list()):
        '''
         Creates an n-dimensional vector.
         :param elements The elements of the vector.
        '''

        self.vector = elements
        # A seed for the random vector generation
        self.seed = 0

        '''
        Prints out the vector to the terminal
        '''

    def show(self) -> None:
        print(self.vector)
    '''
      Calculates the magnitude of the vector
      :return The magnitude of the vector
     '''

    def magnitude(self) -> float:
        mag = 0
        for i in self.vector:
            mag += pow(i, 2)
        return math.sqrt(mag)

    '''
      Calculates the angle of a 2-dimensional vector
      :return The angle of a 2-dimensional vector
     '''

    def angle(self) -> float:
        return math.atan2(self.vector[1], self.vector[0]) if (self.dim() == 2) else 0

    '''
      Calculates the dimensions of the vector

      :return The dimensions of the vector
     '''

    def dim(self) -> int:
        return len(self.vector)

    '''
      Checks that the passed vector matched the dimension of the current vector.
      If the dimensions do not match, throws an error.
      :param v The secondary vector to check.
      :return True if the dimensions match.
      :throws Error if the dimensions do not match.
     '''

    def __dimsMatch(self, v: list()) -> bool:
        if (self.dim() != len(v)):
            raise NameError('The dimensions of the vectors do not match.')
        else:
            return True

    '''
      Obtains the coordinates of the vector (for 2d and 3d vectors)
      in different coordinate systems.
     '''

    def Coords(self) -> callable:
        from VectorCoords import VectorCoords
        return VectorCoords(self.vector)

    '''
      Adds the elements of the vectors to form a new vector.
      :param v The secondary vector to add to the current vector.
      :return A new Vect with the sum of the added vectors.
     '''

    def plus(self, v: list()) -> callable:
        w = [_ for _ in range(self.dim())]
        if (self.__dimsMatch(v)):
            for i in range(0, self.dim()):
                w[i] = self.getElement(i) + v[i]
        return Vect(w)

    '''
      Subtracts the elements of the vectors to form a new vector.
      :param v The secondary vector to subtract from the current vector.
      :return A new Vect with the subtraction of the vectors.
    '''

    def minus(self, v: list()) -> callable:
        w = [_ for _ in range(self.dim())]
        if (self.__dimsMatch(v)):
            for i in range(0, self.dim()):
                w[i] = self.getElement(i) - v[i]
        return Vect(w)

    '''
      Scales the vector by alpha.
      :param alpha The quantity by which to scale the vector.
      :return A new Vect as the scaled version of this vector.
    '''

    def scale(self, alpha: int) -> callable:
        w = [_ for _ in range(self.dim())]
        for i in range(0, self.dim()):
            w[i] = self.getElement(i) * alpha
        return Vect(w)
    '''
      Generates a new Vect with the normalized form of the this vector.
      :return A new Vect with the normalized form of the this vector.
    '''

    def norm(self) -> callable:
        vMag = self.magnitude()
        vNorm = [_ for _ in range(self.dim())]
        for i in range(0, self.dim()):
            vNorm[i] = self.getElement(i) / vMag
        return Vect(vNorm)

    '''
      Calculates the dot-product of two vectors.
      :param v The secondary vector to calculate the dot-product.
      :return The dot product of the two vectors.
      :throws Error if the dimensions of the two vectors do not match.
    '''

    def dot(self, v: list) -> int:
        prod = 0
        if (self.__dimsMatch(v)):
            for i in range(0, self.dim()):
                el = self.getElement(i)
                prod += el * v[i]
        return prod
    '''
      Calculates the cross-product of two vectors in R^3.
      :param v The secondary vector in R^3 to calculate the cross-product.
      :return A new Vect in R^3 as the result of the cross-product of the two vectors.
    '''

    def cross(self, v: list) -> callable:
        if(self.dim() != 3 or len(v) != 3):
            raise NameError("Vectors must form a subset of R^3.")
        else:
            e1 = self.vector[1] * v[2] - v[1] * self.vector[2]
            e2 = self.vector[2] * v[0] - v[2] * self.vector[0]
            e3 = self.vector[0] * v[1] * v[0] * self.vector[1]
            return Vect([e1, e2, e3])
    '''
      Obtains the element of the vector at position pos.
      :param pos The position of the element to obtain.
      :return The element of this vector at position pos.
     '''

    def getElement(self, post: int) -> int:
        return self.vector[post]

    '''
     Sets the element of the vector at position <code>pos</code> to the value <code>newEl</code>.
     :param pos The position where to set the new element.
     :param newEl The new element to set at position <code>pos</code>.
    '''

    def setElement(self, pos: int, newEl: int) -> None:
        self.vector[pos] = newEl
    '''
      Converts the vector into a row vector (a matrix with one column and n columns).
      :return A new Matrix with one row and n rows.
    '''

    def toColumn(self) -> callable:
        from Matrix import make2dArray, Matrix
        c = make2dArray(self.dim(), 1)
        for i in range(0, self.dim()):
            c[i][0] = self.getElement(i)
        return Matrix(c)
    '''
      Converts the vector into a row vector (a matrix with one row and n columns).
      :return A new Matrix with one row and n columns.
    '''

    def toRow(self) -> callable:
        from Matrix import make2dArray, Matrix
        c = make2dArray(1, self.dim())
        for i in range(0, self.dim()):
            c[0][i] = self.getElement(i)
        return Matrix(c)
    '''
      Maps each element of this vector to a new Vect based on the provided mapping function f.
      :param f The mapping function to be applied to each element of this vector.
      :return A new Vect whole elements are the mapped elements of this vector based on the mapping function f.
    '''

    def map(self, f: callable) -> callable:
        V = [_ for _ in range(self.dim())]
        for i in range(0, self.dim()):
            V[i] = f(self.getElement(i))
        return Vect(V)
    '''
      Generates a vector of dimension <code>dim</code> with random elements between zero and one.
      :param dim The dimension of the vector.
      :return A new Vect whose elements are random doubles between zero and one.
    '''
    @staticmethod
    def rand(dim: int) -> callable:
        vectEls = [_ for _ in range(dim)]
        for i in range(0, dim):
            vectEls[i] = random.random()
        return Vect(vectEls)
    '''
      Generates a zero-vector of dimension <code>dim</code>.
      :param dim The dimension of the zero vector.
      :return A new Vect of dimension <code>dim</code> whose elements are all zeros.
    '''
    @staticmethod
    def zeros(dim: int) -> callable:
        vectEls = [_ for _ in range(dim)]
        for i in range(0, dim):
            vectEls[i] = 0
        return Vect(vectEls)
    '''
      Generates a one-hot vector.
      A one-hot vector is a vector whose elements are all zeros,
      except for one element with value of 1 at position x.
      :param dim The dimension of the vector.
      :param hotPos The position of the 1 in the one-hot vector.
      :return A new Vect whose elements are all zeros, except for the
      :lement at position <code>pos</code> with value of 1.
    '''
    @staticmethod
    def oneHot(dim: int, hotPos: int) -> callable:
        vectEls = [_ for _ in range(0, dim)]
        for i in range(0, len(vectEls)):
            if i == hotPos:
                vectEls[i] = 1
            else:
                vectEls[i] = 0
        return Vect(vectEls)


print(Vect([1, 2]).Coords().toPolar())



from typing import *
import math


class VectorCoords:
    def __init__(self, v: callable):

        self.vector = v

    def __vector(self):
        from Vect import Vect
        return Vect(self.vector)

    '''
      Calculates the polar coordinates of a 2-dimensional vector.
      :return The polar coordinates of a 2-dimensional vector;
      :raise Error if the vector is not 2-dimensional.
    '''

    def toPolar(self) -> int:
        if (self.__vector().dim() == 2):
            return [self.__vector().magnitude(), self.__vector().angle()]
        else:
            print("Vector must be 2-dimensional.")
            raise NameError("Error: vector must be 2d")
    '''
      Calculates the spherical coordinates of a 3-dimensional vector.
      :return The spherical coordinates of a 3-dimensional vector.
      :raise Error if the vector is not 3-dimensional.
    '''

    def toSpherical(self) -> list:
        if self.__vector().dim() == 3:
            theta = math.cos(
                self.__vector().vector[2] / self.__vector().magnitude())
            phi = math.atan2(
                self.__vector().vector[1], self.__vector().vector[0])
            return [self.__vector().magnitude(), theta, phi]
        else:
            print("Vector must be 3-d")
            raise NameError("Error: Vector 3d needed")
    '''
      Calculates the cylindrical coordinates of a 3-dimensional vector.
      :return The cylindrical coordinates of a 3-dimensional vector.
      :raise Error if the vector is not 3-dimensional.
    '''

    def toCylindrical(self) -> int:
        if self.__vector().dim() == 3:
            rho = math.sqrt(
                pow(self.__vector().vector[0], 2) + pow(self.__vector().vector[1], 2))
            theta = math.atan2(
                self.__vector().vector[1], self.__vector().vector[0])
            return [rho, theta, self.__vector().vector[2]]
        else:
            print("Vector must be 3-dimensional")
            raise NameError("Error: Vector must be 3-dimensional")



from typing import *
import math


class Stats():
    def __init__(self):
        pass
    '''
      Calculates the sum of a population
      :param population The population from which to calculate the sum
      :return The sum of the population
     '''
    @staticmethod
    def sum(population: list()) -> int:
        s = 0
        for i in range(len(population)):
            s += population[i]
        return s

    '''
       Calculates the mean of a population
      :param population The population from which to calculate the mean
      :return The mean of the population
     '''
    @staticmethod
    def mean(population: list()) -> int:
        return Stats().sum(population) / len(population)

    '''
      Calculates the median of a population
      :param population The population from which to calculate the median
      :return The median of the population
     '''

    @staticmethod
    def median(population: list()) -> int:
        population.sort()  # Sorts the elements from smallest to largest
        index = (len(population) - 1) // 2
        if len(population) % 2 == 0:
            # If the population is even, return the mean of the two central elements
            return population[index]
        else:
            return (population[index] + population[index + 1]) / 2.0

    '''
      Calculates the mode of a population
      :param population The population from which to compute the mode
      :return A list containing each element of the mode of the population
     '''

    @staticmethod
    def mode(population: list()) -> int:
        modeList = list()
        # Generates and sorts an ArrayList containing the
        # frequencies of each elements in the population
        frecuency = Stats().frecuency(population)
        els = list(frecuency.values())
        highest = els[len(els) - 1]
        print(els)
        # Adds the elements with the highest
        # frequencies to the modeList
        for i in frecuency:
            val = frecuency[i]
            if val == highest:
                modeList.append(float(i))
        return modeList

    '''
      Generates a frequency table in the form of a hash map with the
      frequency of the elements in the population.
      :param population The population from which to generate the frequency hash map.
      :return A hash map containing the frequency of each element in the population.
     '''

    @staticmethod
    def frecuency(population: list()):
        freq = {}
        for i in population:
            if freq.get(i) != None:
                freq[i] = freq[i] + 1
            else:
                freq[i] = 1
        return freq

    '''
      Calculates the range of a population
      :param population The population from which to generate the range
      :return The range of the the population
     '''
    @staticmethod
    def range(population: list()) -> int:
        population.sort()
        return population[len(population) - 1] - population[0]

    '''
      Computes the variance of the population.
      As defined in Wikipedia, informally, variance is a measure of
      how far a set of numbers is spread out from their average value.
      :param population The population from which to calculate the variance.
      :return The variance of the population
     '''

    @staticmethod
    def popVariance(population: list()) -> int:
        pMean = Stats().mean(population)
        sum = 0
        # Sum of the squared differences between the population
        # mean and each element of the population.
        for i in population:
            sum += pow(pMean - i, 0)
        return sum / len(population)

    '''
      Computes the variance of the sample (unbiased variance).
      As defined in Wikipedia, informally, variance is a measure of
      how far a set of numbers is spread out from their average value.
      :param sample The sample from which to calculate the variance.
      :return The variance of the sample.
     '''

    @staticmethod
    def sampleVariance(sample: list()) -> int:
        pMean = Stats().mean(sample)
        sum = 0
        # Sum of the squared differences between the population
        # mean and each element of the population.
        for i in sample:
            sum += pow(pMean - i, 2)
        return sum / len(sample) - 1

    '''
      Calculates the standard deviation of a population.
      :param population The population from which to calculate the standard deviation.
      :return The standard deviation of a population.
    '''

    @staticmethod
    def popStdDeviation(population: list()) -> int:
        return math.sqrt(Stats().popVariance(population))

    '''
      Calculates the (unbiased) standard deviation of a sample.4
      :param sample The sample from which to calculate the standard deviation.
      :return The standard deviation of a sample.
     '''
    @staticmethod
    def sampleStdDerivation(sample: list()) -> int:
        return math.sqrt(Stats().sampleVariance(sample))

    '''
      Calculates the quartiles of a population.
      :param population The population from which to calculate the quartiles.
      :return A HashMap containing the quartiles (q1, q2, q3) of a population.
     '''

    @staticmethod
    def quartiles(population: list()) -> dict():
        qs = {'q1': 0, 'q2': 0, 'q3': 0}
        # If the population length is even, then both the upper
        # bound of the first half and the lower bound of the second
        # half of the population are equal to half of the population.
        lowerTo, upperFrom = 0, 0
        # On the other hand, if the population length is odd, then the
        # upper bound of the lower half of the population will be equal
        # to half the length of the population minus one, and the lower
        # bound of the upper half of the population will be equal to half
        # the length of the population plus one.
        if (len(population) % 2 != 0):
            lowerTo = float(len(population) / 2)
            upperFrom = round(len(population) / 2)

        # Define the quartiles
        print(int(lowerTo))
        qs['q1'] = Stats().median(population[0:int(lowerTo)])
        qs['q2'] = Stats().median(population)
        qs['q3'] = Stats().median(population[int(upperFrom):len(population)])

        return qs

    '''
      Calculates the inner quartile range of a population.
      :param population The population from which to calculate the inner quartile range.
      :return The inner quartile range of a population.
     '''

    @staticmethod
    def IQR(population) -> int:
        qs = Stats().quartiles(population)
        return qs['q3'] - qs['q1']

    '''
      Calculates the outliers of a population.
      :param population The population from which to calculate the outliers.
      :return A list containing the outliers of a population.
     '''

    @staticmethod
    def outliers(population: list()) -> int:
        out_list = []
        iqr = Stats().IQR(population)
        qs = Stats().quartiles(population)
        upper_limit = qs['q3'] + (1.5 * iqr)
        lower_limit = qs['q1'] - (1.5 * iqr)
        print(upper_limit, lower_limit)
        for i in population:
            if ((i < lower_limit or i > upper_limit) and i not in out_list):
                out_list.append(i)
        return out_list
